@@ -21,6 +21,7 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <string.h>
 
 #include <glib/gstdio.h>
 
@@ -40,12 +41,16 @@
 #define S(x) x->str, x->len
 
 
+/* forward decl */
+const char *network_mysqld_proto_field_get_typestring(enum enum_field_types type);
+
 typedef struct {
 	MYSQL_FIELD *fielddef;
 
 	union {
 		guint64 i;
 		gchar *s;
+		double d;
 	} data;
 	guint64 data_len;
 
@@ -67,26 +72,39 @@ void network_mysqld_proto_field_free(network_mysqld_proto_field *field) {
 	case MYSQL_TYPE_TIMESTAMP:
 	case MYSQL_TYPE_DATE:
 	case MYSQL_TYPE_DATETIME:
+	case MYSQL_TYPE_TIME:
+	case MYSQL_TYPE_YEAR:
 
 	case MYSQL_TYPE_TINY:
 	case MYSQL_TYPE_SHORT:
 	case MYSQL_TYPE_INT24:
 	case MYSQL_TYPE_LONG:
+	case MYSQL_TYPE_LONGLONG:
 
 	case MYSQL_TYPE_DECIMAL:
 	case MYSQL_TYPE_NEWDECIMAL:
 
 	case MYSQL_TYPE_ENUM:
+	case MYSQL_TYPE_SET:
+
+	case MYSQL_TYPE_DOUBLE:
+	case MYSQL_TYPE_FLOAT:
 		break;
+	case MYSQL_TYPE_BIT:
+	case MYSQL_TYPE_GEOMETRY:
+	case MYSQL_TYPE_TINY_BLOB:
 	case MYSQL_TYPE_BLOB:
+	case MYSQL_TYPE_MEDIUM_BLOB:
+	case MYSQL_TYPE_LONG_BLOB:
 	case MYSQL_TYPE_VARCHAR:
 	case MYSQL_TYPE_VAR_STRING:
 	case MYSQL_TYPE_STRING:
 		if (field->data.s) g_free(field->data.s);
 		break;
 	default:
-		g_message("%s: unknown field_type to free: %d",
+		g_message("%s: unknown field_type '%s' (%d) to free()",
 				G_STRLOC,
+				network_mysqld_proto_field_get_typestring(field->fielddef->type),
 				field->fielddef->type);
 		break;
 	}
@@ -102,8 +120,66 @@ int network_mysqld_proto_field_get(network_packet *packet,
 	guint32 i32;
 	guint64 i64;
 	int err = 0;
+	gchar *s;
 
 	switch ((guchar)field->fielddef->type) {
+	case MYSQL_TYPE_BIT:
+		err = err || network_mysqld_proto_get_string_len(packet, &field->data.s, field->fielddef->max_length);
+		break;
+	case MYSQL_TYPE_FLOAT: /* float4store */
+		s = NULL;
+		err = err || network_mysqld_proto_get_string_len(packet, &s, 4);
+		if (!err) {
+			union {
+				float d;
+				guchar s[4];
+			} shadow;
+
+			guint i;
+
+			memcpy(shadow.s, s, 4);
+
+#if 0
+			/* on BIG-ENDIAN we may have to swap the bytes */
+			for (i = 0; i < 4 / 2; i++) {
+				guchar tmp;
+
+				tmp = shadow.s[i];
+				shadow.s[i] = shadow.s[3 - i];
+				shadow.s[3 - i] = tmp;
+			}
+#endif
+			field->data.d = shadow.d;
+		}
+		g_free(s);
+		break;
+	case MYSQL_TYPE_DOUBLE: /* float8store */
+		s = NULL;
+		err = err || network_mysqld_proto_get_string_len(packet, &s, 8);
+		if (!err) {
+			union {
+				double d;
+				guchar s[8];
+			} shadow;
+
+			guint i;
+
+			memcpy(shadow.s, s, 8);
+
+#if 0
+			/* on BIG-ENDIAN we may have to swap the bytes */
+			for (i = 0; i < 8 / 2; i++) {
+				guchar tmp;
+
+				tmp = shadow.s[i];
+				shadow.s[i] = shadow.s[7 - i];
+				shadow.s[7 - i] = tmp;
+			}
+#endif
+			field->data.d = shadow.d;
+		}
+		g_free(s);
+		break;
 	case MYSQL_TYPE_TIMESTAMP: /* int4store */
 	case MYSQL_TYPE_LONG:
 		err = err || network_mysqld_proto_get_int32(packet, &i32);
@@ -116,6 +192,7 @@ int network_mysqld_proto_field_get(network_packet *packet,
 		break;
 	case MYSQL_TYPE_INT24:     
 	case MYSQL_TYPE_DATE:      /* int3store, a newdate, old-data is 4 byte */
+	case MYSQL_TYPE_TIME:      /* int3store */
 		err = err || network_mysqld_proto_get_int24(packet, &i32);
 		if (!err) field->data.i = i32;
 		break;
@@ -123,10 +200,12 @@ int network_mysqld_proto_field_get(network_packet *packet,
 		err = err || network_mysqld_proto_get_int16(packet, &i16);
 		if (!err) field->data.i = i16;
 		break;
+	case MYSQL_TYPE_YEAR:
 	case MYSQL_TYPE_TINY:     
 		err = err || network_mysqld_proto_get_int8(packet, &i8);
 		if (!err) field->data.i = i8;
 		break;
+	case MYSQL_TYPE_SET:
 	case MYSQL_TYPE_ENUM:
 		switch (field->fielddef->max_length) {
 		case 1:
@@ -144,7 +223,11 @@ int network_mysqld_proto_field_get(network_packet *packet,
 			break;
 		}
 		break;
+	case MYSQL_TYPE_GEOMETRY:
+	case MYSQL_TYPE_TINY_BLOB:
 	case MYSQL_TYPE_BLOB:
+	case MYSQL_TYPE_MEDIUM_BLOB:
+	case MYSQL_TYPE_LONG_BLOB:
 		switch (field->fielddef->max_length) {
 		case 1:
 			err = err || network_mysqld_proto_get_int8(packet, &i8);
@@ -219,44 +302,62 @@ int network_mysqld_proto_field_get(network_packet *packet,
 		break; }
 	default:
 		g_debug_hexdump(G_STRLOC, packet->data->str, packet->data->len);
-		g_error("%s: unknown field-type to fetch: %d",
+		g_error("%s: unknown field-type '%d' to fetch from offset = %04x",
 				G_STRLOC,
-				field->fielddef->type);
+				field->fielddef->type,
+				packet->offset);
 		break;
 	}
 
 	return err ? -1 : 0;
 }
 
+/**
+ * get the field-definitions of columns that are included in this log-event
+ */
 GPtrArray *network_mysqld_proto_fields_new_full(
 		GPtrArray *fielddefs,
+		gchar *used_cols_bits,
+		guint G_GNUC_UNUSED used_cols_bits_len,
 		gchar *null_bits,
 		guint G_GNUC_UNUSED null_bits_len) {
 	GPtrArray *fields;
-	guint i;
+	guint col;
+	guint null_bit;
 
 	fields = g_ptr_array_new();
 
-	for (i = 0; i < fielddefs->len; i++) {
-		MYSQL_FIELD *fielddef = fielddefs->pdata[i];
-		network_mysqld_proto_field *field = network_mysqld_proto_field_new();
+	for (col = 0, null_bit = 0; col < fielddefs->len; col++) {
+		MYSQL_FIELD *fielddef = fielddefs->pdata[col];
 
-		guint byteoffset = i / 8;
-		guint bitoffset = i % 8;
+		guint col_byteoffset = col / 8;
+		guint col_bitoffset = col % 8;
 
-		field->fielddef = fielddef;
-		field->is_null = (null_bits[byteoffset] >> bitoffset) & 0x1;
+		/* check if we have data for this columns in the row-log-event */
+		if ((used_cols_bits[col_byteoffset] >> col_bitoffset) & 0x1) {
+			network_mysqld_proto_field *field = network_mysqld_proto_field_new();
 
-		/* the field is defined as NOT NULL, so the null-bit shouldn't be set */
-		if ((fielddef->flags & NOT_NULL_FLAG) != 0) {
-			if (field->is_null) {
-				g_error("%s: [%d] field is defined as NOT NULL, but nul-bit is set",
-						G_STRLOC,
-						i
-						);
+			guint null_byteoffset = null_bit / 8;
+			guint null_bitoffset = null_bit % 8;
+
+			field->fielddef = fielddef;
+			field->is_null = (null_bits[null_byteoffset] >> null_bitoffset) & 0x1;
+
+			/* the field is defined as NOT NULL, so the null-bit shouldn't be set */
+			if ((fielddef->flags & NOT_NULL_FLAG) != 0) {
+				if (field->is_null) {
+					g_critical("%s: field [%d] is defined as NOT NULL, but nul-bit is set",
+							G_STRLOC,
+							col);
+
+					return NULL;
+				}
 			}
+
+			null_bit++;
+
+			g_ptr_array_add(fields, field);
 		}
-		g_ptr_array_add(fields, field);
 	}
 
 	return fields;
@@ -318,6 +419,9 @@ struct {
 	{ V(UPDATE_ROWS_EVENT ) },
 	{ V(DELETE_ROWS_EVENT ) },
 	{ V(INCIDENT_EVENT) },
+	{ V(HEARTBEAT_LOG_EVENT) },
+	{ V(IGNORABLE_LOG_EVENT) },
+	{ V(ROWS_QUERY_LOG_EVENT) },
 
 #undef V
 	{ 0, NULL }
@@ -344,19 +448,37 @@ struct {
 } field_type_name[] = {
 	{ MYSQL_TYPE_STRING, "CHAR" },
 	{ MYSQL_TYPE_VARCHAR, "VARCHAR" },
+	{ MYSQL_TYPE_VAR_STRING, "VARCHAR /* varstring */" },
+	{ MYSQL_TYPE_TINY_BLOB, "TINYBLOB" },
+	{ MYSQL_TYPE_MEDIUM_BLOB, "MEDIUMBLOB" },
 	{ MYSQL_TYPE_BLOB, "BLOB" },
+	{ MYSQL_TYPE_LONG_BLOB, "LONGBLOB" },
 
 	{ MYSQL_TYPE_TINY, "TINYINT" },
 	{ MYSQL_TYPE_SHORT, "SMALLINT" },
 	{ MYSQL_TYPE_INT24, "MEDIUMINT" },
 	{ MYSQL_TYPE_LONG, "INT" },
+	{ MYSQL_TYPE_LONGLONG, "BIGINT" },
 	{ MYSQL_TYPE_NEWDECIMAL, "DECIMAL" },
 
 	{ MYSQL_TYPE_ENUM, "ENUM" },
+	{ MYSQL_TYPE_SET, "SET" },
 
 	{ MYSQL_TYPE_TIMESTAMP, "TIMESTAMP" },
 	{ MYSQL_TYPE_DATE, "DATE" },
 	{ MYSQL_TYPE_DATETIME, "DATETIME" },
+	{ MYSQL_TYPE_TIME, "TIME" },
+	{ MYSQL_TYPE_YEAR, "YEAR" },
+	{ MYSQL_TYPE_NEWDATE, "DATE /* new */" },
+
+	{ MYSQL_TYPE_FLOAT, "FLOAT" },
+	{ MYSQL_TYPE_DOUBLE, "DOUBLE" },
+	{ MYSQL_TYPE_DECIMAL, "DECIMAL /* old */" },
+
+	{ MYSQL_TYPE_NULL, "NULL" },
+	{ MYSQL_TYPE_BIT, "BIT" },
+
+	{ MYSQL_TYPE_GEOMETRY, "GEOMETRY" },
 
 	{ 0, NULL }
 };
@@ -396,12 +518,25 @@ void network_mysqld_table_print(network_mysqld_table *tbl) {
 		case MYSQL_TYPE_SHORT:
 		case MYSQL_TYPE_INT24:
 		case MYSQL_TYPE_LONG:
+		case MYSQL_TYPE_LONGLONG:
+		case MYSQL_TYPE_DATE:
+		case MYSQL_TYPE_DATETIME:
+		case MYSQL_TYPE_TIME:
+		case MYSQL_TYPE_TIMESTAMP:
+		case MYSQL_TYPE_YEAR:
+
+		case MYSQL_TYPE_TINY_BLOB:
+		case MYSQL_TYPE_BLOB:
+		case MYSQL_TYPE_MEDIUM_BLOB:
+		case MYSQL_TYPE_LONG_BLOB:
 			g_string_append_printf(out, "  field_%d %s %s NULL",
 					i,
 					network_mysqld_proto_field_get_typestring(field->type),
 					field->flags & NOT_NULL_FLAG ? "NOT" : "DEFAULT"
 				 );
 			break;
+		case MYSQL_TYPE_FLOAT:
+		case MYSQL_TYPE_DOUBLE:
 		case MYSQL_TYPE_DECIMAL:
 		case MYSQL_TYPE_NEWDECIMAL:
 			g_string_append_printf(out, "  field_%d %s(%lu, %u) %s NULL",
@@ -429,12 +564,64 @@ void network_mysqld_table_print(network_mysqld_table *tbl) {
 	g_string_free(out, TRUE);
 }
 
+static int network_mysqld_proto_field_append_to_string(GString *out, network_mysqld_proto_field *field) {
+	switch((guchar)field->fielddef->type) {
+	case MYSQL_TYPE_DATE:
+	case MYSQL_TYPE_TIMESTAMP:
+	case MYSQL_TYPE_DATETIME:
+	case MYSQL_TYPE_TIME:
+	case MYSQL_TYPE_YEAR:
+
+	case MYSQL_TYPE_TINY:
+	case MYSQL_TYPE_SHORT:
+	case MYSQL_TYPE_INT24:
+	case MYSQL_TYPE_LONG:
+	case MYSQL_TYPE_LONGLONG:
+	case MYSQL_TYPE_ENUM:
+	case MYSQL_TYPE_SET:
+		g_string_append_printf(out, "%"G_GUINT64_FORMAT, field->data.i);
+		break;
+	case MYSQL_TYPE_VARCHAR:
+	case MYSQL_TYPE_VAR_STRING:
+	case MYSQL_TYPE_STRING:
+		g_string_append_printf(out, "'%s'", field->data.s ? field->data.s : "");
+		break;
+	case MYSQL_TYPE_FLOAT:
+	case MYSQL_TYPE_DOUBLE:
+		g_string_append_printf(out, "%f", field->data.d);
+		break;
+	case MYSQL_TYPE_GEOMETRY:
+		g_string_append_printf(out, "'...(geometry)'");
+		break;
+	case MYSQL_TYPE_TINY_BLOB:
+	case MYSQL_TYPE_BLOB:
+	case MYSQL_TYPE_MEDIUM_BLOB:
+	case MYSQL_TYPE_LONG_BLOB:
+		g_string_append_printf(out, "'...(blob)'");
+		break;
+	case MYSQL_TYPE_BIT:
+		g_string_append_printf(out, "'...(bit)'");
+		break;
+	case MYSQL_TYPE_NEWDECIMAL:
+		g_string_append_printf(out, "'...(decimal)'");
+		break;
+	default:
+		g_error("%s: field-type '%s' (%d) isn't known",
+				G_STRLOC,
+				network_mysqld_proto_field_get_typestring(field->fielddef->type),
+				field->fielddef->type);
+		break;
+	}
+
+	return 0;
+}
+
 /**
  * decode a binlog event
  */
 int network_mysqld_binlog_event_print(network_mysqld_binlog *binlog, 
 		network_mysqld_binlog_event *event) {
-	guint i;
+	guint i, field_ndx;
 	network_mysqld_table *tbl;
 	int err = 0;
 #if 0
@@ -480,12 +667,19 @@ int network_mysqld_binlog_event_print(network_mysqld_binlog *binlog,
 		network_mysqld_table_print(tbl);
 		break; 
 	case FORMAT_DESCRIPTION_EVENT: /* 15 */
+		g_print("-- format-description:\n");
+		g_print("--   file-version: %d\n", event->event.format_event.binlog_version);
+		g_print("--   writer-version: %s\n", event->event.format_event.master_version);
+		g_print("--   created: %d\n", event->event.format_event.created_ts);
+		g_print("--   no. of known events: %"G_GSIZE_FORMAT"\n", event->event.format_event.perm_events_len);
 		break;
 	case INTVAR_EVENT: /* 5 */
 	 	break;
 	case XID_EVENT: /* 16 */
+		g_print("COMMIT /* xid = %"G_GUINT64_FORMAT" */\n", event->event.xid.xid_id);
 		break;
 	case ROTATE_EVENT: /* 4 */
+		g_print("-- rotating to %s, pos %u\n", event->event.rotate_event.binlog_file,  event->event.rotate_event.binlog_pos);
 		break;
 	case WRITE_ROWS_EVENT:
 	case UPDATE_ROWS_EVENT:
@@ -520,13 +714,20 @@ int network_mysqld_binlog_event_print(network_mysqld_binlog *binlog,
 			err = err || network_mysqld_proto_get_string_len(
 					&row_packet, 
 					&pre_bits,
-					event->event.row_event.null_bits_len);
+					event->event.row_event.null_bits_before_len);
 
 			if (err) break;
 
-			pre_fields = network_mysqld_proto_fields_new_full(tbl->fields, 
+			pre_fields = network_mysqld_proto_fields_new_full(tbl->fields,
+					event->event.row_event.used_columns_before,
+					event->event.row_event.used_columns_before_len,
 					pre_bits, 
-					event->event.row_event.null_bits_len);
+					event->event.row_event.null_bits_before_len);
+
+			if (NULL == pre_fields) {
+				err = 1;
+				break;
+			}
 
 			if (network_mysqld_proto_fields_get(&row_packet, pre_fields)) {
 				break;
@@ -536,12 +737,22 @@ int network_mysqld_binlog_event_print(network_mysqld_binlog *binlog,
 				err = err || network_mysqld_proto_get_string_len(
 						&row_packet, 
 						&post_bits,
-						event->event.row_event.null_bits_len);
+						event->event.row_event.null_bits_after_len);
+
+				if (err) break;
 		
 				post_fields = network_mysqld_proto_fields_new_full(tbl->fields, 
+					event->event.row_event.used_columns_after,
+					event->event.row_event.used_columns_after_len,
 					post_bits, 
-					event->event.row_event.null_bits_len);
-				network_mysqld_proto_fields_get(&row_packet, post_fields);
+					event->event.row_event.null_bits_after_len);
+				if (NULL == post_fields) {
+					err = 1;
+					break;
+				}
+				if (network_mysqld_proto_fields_get(&row_packet, post_fields)) {
+					break;
+				}
 			}
 
 			/* call lua */
@@ -552,92 +763,70 @@ int network_mysqld_binlog_event_print(network_mysqld_binlog *binlog,
 						tbl->db_name->str,
 						tbl->table_name->str);
 
-				for (i = 0; i < post_fields->len; i++) {
-					network_mysqld_proto_field *field = post_fields->pdata[i];
-					if (i > 0) {
-						g_string_append_printf(out, ", ");
-					}
-					if (field->is_null) {
-						g_string_append_printf(out, "field_%d = NULL", i);
-					} else {
-						switch((guchar)field->fielddef->type) {
-						case MYSQL_TYPE_DATE:
-						case MYSQL_TYPE_TIMESTAMP:
-						case MYSQL_TYPE_DATETIME:
+				for (i = 0, field_ndx = 0; i < tbl->fields->len; i++) {
+					guint col_byteoffset = i / 8;
+					guint col_bitoffset = i % 8;
 
-						case MYSQL_TYPE_TINY:
-						case MYSQL_TYPE_SHORT:
-						case MYSQL_TYPE_INT24:
-						case MYSQL_TYPE_LONG:
-						case MYSQL_TYPE_ENUM:
-							g_string_append_printf(out, "field_%d = %"G_GUINT64_FORMAT, i, field->data.i);
-							break;
-						case MYSQL_TYPE_VARCHAR:
-						case MYSQL_TYPE_VAR_STRING:
-						case MYSQL_TYPE_STRING:
-							g_string_append_printf(out, "field_%d = '%s'", i, field->data.s ? field->data.s : "");
-							break;
-						case MYSQL_TYPE_BLOB:
-							g_string_append_printf(out, "field_%d = '...(blob)'", i);
-							break;
-						case MYSQL_TYPE_NEWDECIMAL:
-							g_string_append_printf(out, "'...(decimal)'");
-							break;
-						default:
-							g_error("%s: field-type %d isn't known",
-									G_STRLOC,
-									field->fielddef->type);
-							break;
+					if ((event->event.row_event.used_columns_after[col_byteoffset] >> col_bitoffset) & 0x1) {
+						network_mysqld_proto_field *field = post_fields->pdata[field_ndx++];
+
+						if (field_ndx > 1) {
+							g_string_append_printf(out, ", ");
+						}
+						g_string_append_printf(out, "field_%d ", i);
+
+						if (field->is_null) {
+							g_string_append(out, "= NULL");
+						} else {
+							g_string_append(out, "= ");
+							network_mysqld_proto_field_append_to_string(out, field);
 						}
 					}
 				}
 
 				g_string_append_printf(out, "\n WHERE ");
-				for (i = 0; i < pre_fields->len; i++) {
-					network_mysqld_proto_field *field = pre_fields->pdata[i];
-					if (i > 0) {
-						g_string_append_printf(out, " AND ");
-					}
-					if (field->is_null) {
-						g_string_append_printf(out, "field_%d IS NULL", i);
-					} else {
-						switch((guchar)field->fielddef->type) {
-						case MYSQL_TYPE_TIMESTAMP:
-						case MYSQL_TYPE_DATE:
-						case MYSQL_TYPE_DATETIME:
 
-						case MYSQL_TYPE_TINY:
-						case MYSQL_TYPE_SHORT:
-						case MYSQL_TYPE_INT24:
-						case MYSQL_TYPE_LONG:
-						case MYSQL_TYPE_ENUM:
-							g_string_append_printf(out, "field_%d = %"G_GUINT64_FORMAT, i, field->data.i);
-							break;
-						case MYSQL_TYPE_VARCHAR:
-						case MYSQL_TYPE_VAR_STRING:
-						case MYSQL_TYPE_STRING:
-							g_string_append_printf(out, "field_%d = '%s'", i, field->data.s ? field->data.s : "");
-							break;
-						case MYSQL_TYPE_BLOB:
-							g_string_append_printf(out, "field_%d = '...(blob)'", i);
-							break;
-						case MYSQL_TYPE_NEWDECIMAL:
-							g_string_append_printf(out, "'...(decimal)'");
-							break;
-						default:
-							g_error("%s: field-type %d isn't known",
-									G_STRLOC,
-									field->fielddef->type);
-							break;
+				for (i = 0, field_ndx = 0; i < tbl->fields->len; i++) {
+					guint col_byteoffset = i / 8;
+					guint col_bitoffset = i % 8;
+
+					if ((event->event.row_event.used_columns_before[col_byteoffset] >> col_bitoffset) & 0x1) {
+						network_mysqld_proto_field *field = pre_fields->pdata[field_ndx++];
+						if (field_ndx > 1) {
+							g_string_append_printf(out, " AND ");
+						}
+
+						g_string_append_printf(out, "field_%d ", i);
+						if (field->is_null) {
+							g_string_append(out, "IS NULL");
+						} else {
+							g_string_append(out, "= ");
+							network_mysqld_proto_field_append_to_string(out, field);
 						}
 					}
 				}
 				break;
 			case WRITE_ROWS_EVENT:
-				g_string_append_printf(out, "INSERT INTO %s.%s VALUES\n  (",
+				g_string_append_printf(out, "INSERT INTO %s.%s ",
 						tbl->db_name->str,
 						tbl->table_name->str);
 
+				/* ... get the column-index right */
+				g_string_append(out, "(");
+				for (i = 0; i < tbl->fields->len; i++) {
+					guint col_byteoffset = i / 8;
+					guint col_bitoffset = i % 8;
+
+					if ((event->event.row_event.used_columns_before[col_byteoffset] >> col_bitoffset) & 0x1) {
+						if (out->str[out->len - 1] != '(') g_string_append(out, ", ");
+
+						g_string_append_printf(out, "field_%d", i);
+					}
+				}
+				g_string_append(out, ")");
+
+				g_string_append(out, " VALUES\n  (");
+
 				for (i = 0; i < pre_fields->len; i++) {
 					network_mysqld_proto_field *field = pre_fields->pdata[i];
 					if (i > 0) {
@@ -646,35 +835,7 @@ int network_mysqld_binlog_event_print(network_mysqld_binlog *binlog,
 					if (field->is_null) {
 						g_string_append(out, "NULL");
 					} else {
-						switch((guchar)field->fielddef->type) {
-						case MYSQL_TYPE_TIMESTAMP:
-						case MYSQL_TYPE_DATE:
-						case MYSQL_TYPE_DATETIME:
-
-						case MYSQL_TYPE_TINY:
-						case MYSQL_TYPE_SHORT:
-						case MYSQL_TYPE_INT24:
-						case MYSQL_TYPE_LONG:
-						case MYSQL_TYPE_ENUM:
-							g_string_append_printf(out, "%"G_GUINT64_FORMAT, field->data.i);
-							break;
-						case MYSQL_TYPE_VARCHAR:
-						case MYSQL_TYPE_VAR_STRING:
-						case MYSQL_TYPE_STRING:
-							g_string_append_printf(out, "'%s'", field->data.s ? field->data.s : "");
-							break;
-						case MYSQL_TYPE_BLOB:
-							g_string_append_printf(out, "'...(blob)'");
-							break;
-						case MYSQL_TYPE_NEWDECIMAL:
-							g_string_append_printf(out, "'...(decimal)'");
-							break;
-						default:
-							g_error("%s: field-type %d isn't known",
-									G_STRLOC,
-									field->fielddef->type);
-							break;
-						}
+						network_mysqld_proto_field_append_to_string(out, field);
 					}
 				}
 
@@ -685,42 +846,21 @@ int network_mysqld_binlog_event_print(network_mysqld_binlog *binlog,
 						tbl->db_name->str,
 						tbl->table_name->str);
 
-				for (i = 0; i < pre_fields->len; i++) {
-					network_mysqld_proto_field *field = pre_fields->pdata[i];
-					if (i > 0) {
-						g_string_append_printf(out, " AND ");
-					}
-					if (field->is_null) {
-						g_string_append_printf(out, "field_%d IS NULL", i);
-					} else {
-						switch((guchar)field->fielddef->type) {
-						case MYSQL_TYPE_TIMESTAMP:
-						case MYSQL_TYPE_DATE:
-						case MYSQL_TYPE_DATETIME:
+				for (i = 0, field_ndx = 0; i < tbl->fields->len; i++) {
+					guint col_byteoffset = i / 8;
+					guint col_bitoffset = i % 8;
 
-						case MYSQL_TYPE_TINY:
-						case MYSQL_TYPE_SHORT:
-						case MYSQL_TYPE_INT24:
-						case MYSQL_TYPE_LONG:
-						case MYSQL_TYPE_ENUM:
-							g_string_append_printf(out, "field_%d = %"G_GUINT64_FORMAT, i, field->data.i);
-							break;
-						case MYSQL_TYPE_VARCHAR:
-						case MYSQL_TYPE_VAR_STRING:
-						case MYSQL_TYPE_STRING:
-							g_string_append_printf(out, "field_%d = '%s'", i, field->data.s ? field->data.s : "");
-							break;
-						case MYSQL_TYPE_BLOB:
-							g_string_append_printf(out, "field_%d = '...(blob)'", i);
-							break;
-						case MYSQL_TYPE_NEWDECIMAL:
-							g_string_append_printf(out, "'...(decimal)'");
-							break;
-						default:
-							g_error("%s: field-type %d isn't known",
-									G_STRLOC,
-									field->fielddef->type);
-							break;
+					if ((event->event.row_event.used_columns_before[col_byteoffset] >> col_bitoffset) & 0x1) {
+						network_mysqld_proto_field *field = pre_fields->pdata[field_ndx++];
+						if (field_ndx > 1) {
+							g_string_append_printf(out, " AND ");
+						}
+						g_string_append_printf(out, "field_%d ", i);
+						if (field->is_null) {
+							g_string_append(out, "IS NULL");
+						} else {
+							g_string_append(out, "= ");
+							network_mysqld_proto_field_append_to_string(out, field);
 						}
 					}
 				}
@@ -729,9 +869,8 @@ int network_mysqld_binlog_event_print(network_mysqld_binlog *binlog,
 			default:
 				break;
 			}
-#if 0
-			g_print("-- %s:\n%s\n\n", G_STRLOC, out->str);
-#endif
+
+			g_print("%s\n\n", out->str);
 
 			g_string_free(out, TRUE);
 
@@ -741,9 +880,21 @@ int network_mysqld_binlog_event_print(network_mysqld_binlog *binlog,
 			if (post_bits) g_free(post_bits);
 		} while (row_packet.data->len > row_packet.offset);
 
-		g_assert_cmpint(row_packet.data->len, ==, row_packet.offset);
+		if (0 == err) {
+			if (row_packet.offset != row_packet.data->len) {
+				g_debug("%s: event_type %d: offset = %d, length = %"G_GSIZE_FORMAT,
+						G_STRLOC,
+						event->event_type,
+						row_packet.offset,
+						row_packet.data->len);
+			}
+		}
 
 		break; }
+	case ROWS_QUERY_LOG_EVENT:
+		g_print("-- next RBR query: %s\n", event->event.rows_query.query);
+		break;
+
 	default:
 		g_message("%s: unknown event-type: %d",
 				G_STRLOC,
@@ -759,7 +910,8 @@ int network_mysqld_binlog_event_print(network_mysqld_binlog *binlog,
 int replicate_binlog_dump_file(
 		const char *filename, 
 		gint startpos,
-		gboolean find_startpos
+		gboolean find_startpos,
+		gint stoppos
 		) {
 	int fd;
 	char binlog_header[4];
@@ -872,7 +1024,7 @@ int replicate_binlog_dump_file(
 	packet->offset = 0;
 
 	/* next are the events, without the mysql packet header */
-	while (19 == (packet->data->len = read(fd, packet->data->str, 19))) {
+	while (19 == (packet->data->len = read(fd, packet->data->str, 19)) && (stoppos <= 0 || binlog_pos < stoppos)) {
 		gssize len;
 		packet->data->str[packet->data->len] = '\0'; /* term the string */
 
@@ -891,8 +1043,7 @@ int replicate_binlog_dump_file(
 			break;
 		}
 
-		g_print("-- %s: (--binlog-start-pos=%ld (next event at %"G_GUINT32_FORMAT")) event = %s (%d)\n",
-				G_STRLOC,
+		g_print("-- (--binlog-start-pos=%ld (next event at %"G_GUINT32_FORMAT")) event = %s (%d)\n",
 				binlog_pos,
 				event->log_pos,
 				network_mysqld_binlog_get_eventname(event->event_type),
@@ -961,6 +1112,7 @@ int main(int argc, char **argv) {
 	GKeyFile *keyfile = NULL;
 	chassis_log *log;
 	gint binlog_start_pos = 0;
+	gint binlog_stop_pos = 0;
 	gboolean binlog_find_start_pos = FALSE;
 
 	/* can't appear in the configfile */
@@ -980,6 +1132,7 @@ int main(int argc, char **argv) {
 		
 		{ "binlog-file",              0, 0, G_OPTION_ARG_FILENAME, NULL, "binlog filename", NULL },
 		{ "binlog-start-pos",         0, 0, G_OPTION_ARG_INT, NULL, "binlog start position", NULL },
+		{ "binlog-stop-pos",          0, 0, G_OPTION_ARG_INT, NULL, "binlog stop position", NULL },
 		{ "binlog-find-start-pos",    0, 0, G_OPTION_ARG_NONE, NULL, "find binlog start position", NULL },
 		
 		{ NULL,                       0, 0, G_OPTION_ARG_NONE,   NULL, NULL, NULL }
@@ -1042,6 +1195,7 @@ int main(int argc, char **argv) {
 	main_entries[i++].arg_data  = &(log->use_syslog);
 	main_entries[i++].arg_data  = &(binlog_filename);
 	main_entries[i++].arg_data  = &(binlog_start_pos);
+	main_entries[i++].arg_data  = &(binlog_stop_pos);
 	main_entries[i++].arg_data  = &(binlog_find_start_pos);
 
 	option_ctx = g_option_context_new("- MySQL Binlog Dump");
@@ -1137,7 +1291,8 @@ int main(int argc, char **argv) {
 	replicate_binlog_dump_file(
 			binlog_filename,
 			binlog_start_pos,
-			binlog_find_start_pos
+			binlog_find_start_pos,
+			binlog_stop_pos
 			);
 
 exit_nicely:
