@@ -24,6 +24,9 @@
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
+
+#include <stdio.h>
+
 /**
  * replication 
  */
@@ -139,10 +142,16 @@ int network_mysqld_proto_get_binlog_event_header(network_packet *packet, network
 }
 
 int network_mysqld_proto_get_binlog_event(network_packet *packet, 
-		network_mysqld_binlog G_GNUC_UNUSED *binlog,
+		network_mysqld_binlog *binlog,
 		network_mysqld_binlog_event *event) {
 
 	int err = 0;
+	int maj, min, pat;
+
+	if (binlog->checksum == NETWORK_MYSQLD_BINLOG_CHECKSUM_CRC32) {
+		/* patch the packet-len for the decoders as if there would be no checksum */
+		packet->data->len -= 4;
+	}
 
 	switch ((guchar)event->event_type) {
 	case QUERY_EVENT:
@@ -197,12 +206,43 @@ int network_mysqld_proto_get_binlog_event(network_packet *packet,
 		err = err || network_mysqld_proto_get_int8(packet, &event->event.format_event.log_header_len);
 		g_assert_cmpint(event->event.format_event.log_header_len, ==, 19);
 
-		/* there is some funky event-permutation going on */
-		event->event.format_event.perm_events_len = packet->data->len - packet->offset;
+		/* decode the server-version string into a integer */
+		err = err || (3 != sscanf(event->event.format_event.master_version, "%d.%d.%d%*s", &maj, &min, &pat));
+		err = err || (maj > 100 || maj < 0);
+		err = err || (min > 100 || min < 0);
+		err = err || (pat > 100 || pat < 0);
+
+		if ((maj << 16) + (min << 8) + (pat << 0) >= 0x050602) {
+			event->event.format_event.perm_events_len = packet->data->len - packet->offset;
+			g_assert_cmpint(event->event.format_event.perm_events_len, ==, ENUM_END_EVENT - 1 + 1 + 4);
+
+			event->event.format_event.perm_events_len -= 1 + 4; /* FIXME: check how it is implemented without CRC32 */
+		} else {
+			event->event.format_event.perm_events_len = packet->data->len - packet->offset;
+		}
+		/* event-length, one byte for each event */
 		err = err || network_mysqld_proto_get_string_len(
 				packet, 
 				&event->event.format_event.perm_events,
-				packet->data->len - packet->offset);
+				event->event.format_event.perm_events_len);
+
+		/* check if we have checksums */
+		if ((maj << 16) + (min << 8) + (pat << 0) >= 0x050602) {
+			guint8 checksum;
+
+			g_assert_cmpint(packet->data->len - packet->offset, ==, 1 + 4);
+
+			err = err || network_mysqld_proto_get_int8(packet, &checksum);
+
+			binlog->checksum = checksum;
+
+			/* leave the last 4 bytes (CRC32 checksum) in the buffer */
+		}
+
+		if (binlog->checksum == NETWORK_MYSQLD_BINLOG_CHECKSUM_CRC32) {
+			/* patch the packet len to only cover the event without the CRC32 checksum */
+			packet->data->len -= 4;
+		}
 		
 		break;
 	case USER_VAR_EVENT:
@@ -289,7 +329,9 @@ int network_mysqld_proto_get_binlog_event(network_packet *packet,
 		break;
 	case DELETE_ROWS_EVENT: /* 25 */
 	case UPDATE_ROWS_EVENT: /* 24 */
-	case WRITE_ROWS_EVENT:  /* 23 */
+	case WRITE_ROWS_EVENT: { /* 23 */
+		guint col, cols_before_present, cols_after_present;
+
 		err = err || network_mysqld_proto_get_int48(packet,
 				&event->event.row_event.table_id); /* 6 bytes */
 		err = err || network_mysqld_proto_get_int16(packet,
@@ -299,19 +341,41 @@ int network_mysqld_proto_get_binlog_event(network_packet *packet,
 				&event->event.row_event.columns_len);
 
 		/* a bit-mask of used-fields (m_cols.bitmap) */
-		event->event.row_event.used_columns_len = (int)((event->event.row_event.columns_len+7)/8);
+		event->event.row_event.used_columns_before_len = (int)((event->event.row_event.columns_len+7)/8);
 		err = err || network_mysqld_proto_get_string_len(
 				packet,
-				&event->event.row_event.used_columns,
-				event->event.row_event.used_columns_len);
+				&event->event.row_event.used_columns_before,
+				event->event.row_event.used_columns_before_len);
 
 		if (event->event_type == UPDATE_ROWS_EVENT) {
-			/* the before image */
-			err = err || network_mysqld_proto_skip(packet, event->event.row_event.used_columns_len);
+			/* used columns in the after image */
+			event->event.row_event.used_columns_after_len = (int)((event->event.row_event.columns_len+7)/8);
+			err = err || network_mysqld_proto_get_string_len(
+					packet,
+					&event->event.row_event.used_columns_after,
+					event->event.row_event.used_columns_after_len);
+		}
+
+		/* we only have as many NULL-bits as we have columns that can be nullable */
+		for (col = 0, cols_before_present = 0, cols_after_present = 0;
+				col < event->event.row_event.columns_len;
+				col++) {
+			int byte_ndx = col / 8;
+			int bit_ndx  = col % 8;
+
+			if ((event->event.row_event.used_columns_before[byte_ndx] >> bit_ndx) & 1) {
+				cols_before_present++;
+			}
+			if (event->event_type == UPDATE_ROWS_EVENT) {
+				if ((event->event.row_event.used_columns_after[byte_ndx] >> bit_ndx) & 1) {
+					cols_after_present++;
+				}
+			}
 		}
 
 		/* null-bits for all the columns */
-		event->event.row_event.null_bits_len = (int)((event->event.row_event.columns_len+7)/8);
+		event->event.row_event.null_bits_before_len = (int)((cols_before_present+7)/8);
+		event->event.row_event.null_bits_after_len = (int)((cols_after_present+7)/8);
 
 		/* the null-bits + row,
 		 *
@@ -324,7 +388,7 @@ int network_mysqld_proto_get_binlog_event(network_packet *packet,
 				&event->event.row_event.row,
 				event->event.row_event.row_len);
 		
-		break;
+		break; }
 	case INTVAR_EVENT:
 		err = err || network_mysqld_proto_get_int8(packet,
 				&event->event.intvar.type);
@@ -336,6 +400,13 @@ int network_mysqld_proto_get_binlog_event(network_packet *packet,
 		err = err || network_mysqld_proto_get_int64(packet,
 				&event->event.xid.xid_id);
 		break;
+	case ROWS_QUERY_LOG_EVENT:
+		err = err || network_mysqld_proto_get_int8(packet, &event->event.rows_query.query_len); /* a length byte */
+		err = err || network_mysqld_proto_get_string_len(
+				packet, 
+				&event->event.rows_query.query,
+				packet->data->len - packet->offset);
+		break;
 	default:
 		g_critical("%s: unhandled binlog-event: %d", 
 				G_STRLOC, 
@@ -343,7 +414,23 @@ int network_mysqld_proto_get_binlog_event(network_packet *packet,
 		return -1;
 	}
 
-	/* FIXME: we should check if we have handled all bytes */
+	if (binlog->checksum == NETWORK_MYSQLD_BINLOG_CHECKSUM_CRC32) {
+		/* patch the packet-len back */
+		packet->data->len += 4;
+
+		/* the next 4 bytes are CRC32 if checksum is CRC32 */
+		err = err || network_mysqld_proto_skip(packet, 4);
+	}
+
+	/* check if we have handled all bytes */
+	if (packet->offset != packet->data->len) {
+		g_debug("%s: event_type %d: offset = %d, length = %"G_GSIZE_FORMAT,
+				G_STRLOC,
+				event->event_type,
+				packet->offset,
+				packet->data->len);
+		return -1;
+	}
 
 	return err ? -1 : 0;
 }
@@ -377,9 +464,13 @@ void network_mysqld_binlog_event_free(network_mysqld_binlog_event *event) {
 	case DELETE_ROWS_EVENT:
 	case UPDATE_ROWS_EVENT:
 	case WRITE_ROWS_EVENT:
-		if (event->event.row_event.used_columns) g_free(event->event.row_event.used_columns);
+		if (event->event.row_event.used_columns_before) g_free(event->event.row_event.used_columns_before);
+		if (event->event.row_event.used_columns_after) g_free(event->event.row_event.used_columns_after);
 		if (event->event.row_event.row) g_free(event->event.row_event.row);
 		break;
+	case ROWS_QUERY_LOG_EVENT:
+		if (event->event.rows_query.query) g_free(event->event.rows_query.query);
+		break;
 	default:
 		break;
 	}
@@ -487,12 +578,15 @@ int network_mysqld_binlog_event_tablemap_get(
 					&byte0);
 			err = err || network_mysqld_proto_get_int8(&metadata_packet, 
 					&byte1);
-			field->max_length = byte1;
-
-			if (!err && ((byte0 & 0x30) != 0x30)) {
-				/* a long CHAR() field */
-				field->max_length |= (((byte0 & 0x30) ^ 0x30) << 4);
-				field->type = byte0 | 0x30; /* see #37426 */
+			if (!err) {
+				if ((byte0 & 0x30) != 0x30) {
+					/* a long CHAR() field */
+					field->max_length |= (((byte0 & 0x30) ^ 0x30) << 4);
+					field->type = byte0 | 0x30; /* see #37426 */
+				} else {
+					field->max_length = byte1;
+					field->type       = byte0;
+				}
 			}
 
 			break;
@@ -507,16 +601,45 @@ int network_mysqld_binlog_event_tablemap_get(
 				field->max_length = varchar_length;
 			}
 			break;
+		case MYSQL_TYPE_GEOMETRY: /* 255 */
 		case MYSQL_TYPE_BLOB: /* 252 */
-			field->type = col_type;
-
 			/* the packlength (1 .. 4) */
 			err = err || network_mysqld_proto_get_int8(&metadata_packet, &byte0);
 
-			field->max_length = byte0;
+			if (!err) {
+				field->type = (guchar)col_type;
+				field->max_length = byte0;
+
+				/* fix the field-type */
+				if (field->type == MYSQL_TYPE_BLOB) {
+					switch (field->max_length) {
+					case 1:
+						field->type = MYSQL_TYPE_TINY_BLOB;
+						break;
+					case 2:
+						field->type = MYSQL_TYPE_BLOB;
+						break;
+					case 3:
+						field->type = MYSQL_TYPE_MEDIUM_BLOB;
+						break;
+					case 4:
+						field->type = MYSQL_TYPE_LONG_BLOB;
+						break;
+					default:
+						err = 1;
+						break;
+					}
+				} else if (field->type == MYSQL_TYPE_GEOMETRY) {
+					if (field->max_length != 4) {
+						err = 1;
+					}
+				} else {
+					g_assert_not_reached();
+				}
+			}
+
 			break;
 		case MYSQL_TYPE_NEWDECIMAL:
-		case MYSQL_TYPE_DECIMAL:
 			field->type = col_type;
 			/**
 			 * byte 0: precisions
@@ -553,16 +676,34 @@ int network_mysqld_binlog_event_tablemap_get(
 			}
 			break;
 		case MYSQL_TYPE_BIT:
-			err = err || network_mysqld_proto_skip(&metadata_packet, 2);
+			/*
+			 * [0] - bitlength % 8
+			 * [1] - bitlength / 8
+			 */
+			field->type = col_type;
+
+			err = err || network_mysqld_proto_get_int8(&metadata_packet, &byte0);
+			err = err || network_mysqld_proto_get_int8(&metadata_packet, &byte1);
+
+			if (!err) {
+				field->max_length = byte1 + (byte0 == 0 ? 0 : 1);
+			}
+
 			break;
-		case MYSQL_TYPE_DATE:
+
+		/* no metadata */
+		case MYSQL_TYPE_DECIMAL:
+		case MYSQL_TYPE_DATE: 
 		case MYSQL_TYPE_DATETIME:
+		case MYSQL_TYPE_TIME:
 		case MYSQL_TYPE_TIMESTAMP:
+		case MYSQL_TYPE_YEAR:
 
 		case MYSQL_TYPE_TINY:
 		case MYSQL_TYPE_SHORT:
 		case MYSQL_TYPE_INT24:
 		case MYSQL_TYPE_LONG:
+		case MYSQL_TYPE_LONGLONG:
 			field->type = col_type;
 			break;
 		default:
