@@ -111,11 +111,14 @@ NETWORK_API int network_mysqld_proto_get_gstring(network_packet *packet, GString
 NETWORK_API int network_mysqld_proto_peek_lenenc_type(network_packet *packet, network_mysqld_lenenc_type *type);
 NETWORK_API int network_mysqld_proto_get_lenenc_int(network_packet *packet, guint64 *v);
 
-NETWORK_API MYSQL_FIELD *network_mysqld_proto_fielddef_new(void);
-NETWORK_API void network_mysqld_proto_fielddef_free(MYSQL_FIELD *fielddef);
+typedef MYSQL_FIELD network_mysqld_proto_fielddef_t;
+NETWORK_API network_mysqld_proto_fielddef_t *network_mysqld_proto_fielddef_new(void);
+NETWORK_API void network_mysqld_proto_fielddef_free(network_mysqld_proto_fielddef_t *fielddef);
+NETWORK_API int network_mysqld_proto_get_fielddef(network_packet *packet, network_mysqld_proto_fielddef_t *field, guint32 capabilities);
 
-NETWORK_API GPtrArray *network_mysqld_proto_fielddefs_new(void);
-NETWORK_API void network_mysqld_proto_fielddefs_free(GPtrArray *fielddefs);
+typedef GPtrArray network_mysqld_proto_fielddefs_t;
+NETWORK_API network_mysqld_proto_fielddefs_t *network_mysqld_proto_fielddefs_new(void);
+NETWORK_API void network_mysqld_proto_fielddefs_free(network_mysqld_proto_fielddefs_t *fielddefs);
 
 NETWORK_API guint32 network_mysqld_proto_get_packet_len(GString *_header);
 NETWORK_API guint8 network_mysqld_proto_get_packet_id(GString *_header);
