@@ -145,10 +145,6 @@ DEFAULT_SCRIPT_FILENAME	 = "/tmp/dummy.lua"
 default_proxy_options = {
 	["proxy-backend-addresses"] = MYSQL_HOST .. ":" .. MYSQL_PORT,
 	["proxy-address"]		   	= PROXY_HOST .. ":" .. PROXY_PORT,
-	["admin-address"]		   	= PROXY_HOST .. ":" .. ADMIN_PORT,
-	["admin-username"]		   	= ADMIN_USER,
-	["admin-password"]		   	= ADMIN_PASSWORD,
-	["admin-lua-script"]			= ADMIN_DEFAULT_SCRIPT_FILENAME,
 	["pid-file"]				= PROXY_PIDFILE,
 	["proxy-lua-script"]		= DEFAULT_SCRIPT_FILENAME,
 	["plugin-dir"]			= PROXY_LIBPATH,
@@ -158,10 +154,6 @@ default_proxy_options = {
 default_master_options = {
 	["proxy-backend-addresses"] = MYSQL_HOST .. ":" .. MYSQL_PORT,
 	["proxy-address"]		   	= PROXY_HOST .. ":" .. PROXY_MASTER_PORT,
-	["admin-address"]		   	= PROXY_HOST .. ":" .. ADMIN_MASTER_PORT,
-	["admin-username"]		   	= ADMIN_USER,
-	["admin-password"]		   	= ADMIN_PASSWORD,
-	["admin-lua-script"]			= ADMIN_DEFAULT_SCRIPT_FILENAME,
 	["pid-file"]				= PROXY_MASTER_PIDFILE,
 	["proxy-lua-script"]		= DEFAULT_SCRIPT_FILENAME,
 	["plugin-dir"]			= PROXY_LIBPATH,
@@ -171,10 +163,6 @@ default_master_options = {
 default_slave_options = {
 	["proxy-backend-addresses"] = MYSQL_HOST .. ":" .. MYSQL_PORT,
 	["proxy-address"]		   	= PROXY_HOST .. ":" .. PROXY_SLAVE_PORT,
-	["admin-address"]		  	= PROXY_HOST .. ":" .. ADMIN_SLAVE_PORT,
-	["admin-username"]		   	= ADMIN_USER,
-	["admin-password"]		   	= ADMIN_PASSWORD,
-	["admin-lua-script"]			= ADMIN_DEFAULT_SCRIPT_FILENAME,
 	["pid-file"]				= PROXY_SLAVE_PIDFILE,
 	["proxy-lua-script"]		= DEFAULT_SCRIPT_FILENAME,
 	["plugin-dir"]			= PROXY_LIBPATH,
@@ -763,10 +751,6 @@ function chain_proxy (backend_lua_scripts, second_lua_script, use_replication)
 		backend_proxy_options = {
 			["proxy-backend-addresses"] = MYSQL_HOST .. ":" .. MYSQL_PORT,
 			["proxy-address"]		   = backend_addresses[i],
-			["admin-address"]		   = PROXY_HOST .. ":" .. (ADMIN_CHAIN_PORT + i - 1),
-			["admin-username"]		   	= ADMIN_USER,
-			["admin-password"]		   	= ADMIN_PASSWORD,
-			["admin-lua-script"]		= ADMIN_DEFAULT_SCRIPT_FILENAME,
 			["pid-file"]				= PROXY_CHAIN_PIDFILE .. i,
 			["proxy-lua-script"]		= backend_lua_script or DEFAULT_SCRIPT_FILENAME,
 			["plugin-dir"]			= PROXY_LIBPATH,
@@ -788,10 +772,6 @@ function chain_proxy (backend_lua_scripts, second_lua_script, use_replication)
 	second_proxy_options = {
 			["proxy-backend-addresses"] = backend_addresses ,
 			["proxy-address"]		   	= PROXY_HOST .. ":" .. PROXY_PORT,
-			["admin-address"]		   	= PROXY_HOST .. ":" .. ADMIN_PORT,
-			["admin-username"]		   	= ADMIN_USER,
-			["admin-password"]		   	= ADMIN_PASSWORD,
-			["admin-lua-script"]		= ADMIN_DEFAULT_SCRIPT_FILENAME,
 			["pid-file"]				= PROXY_PIDFILE,
 			["proxy-lua-script"]		= second_lua_script or DEFAULT_SCRIPT_FILENAME,
 			["plugin-dir"]			= PROXY_LIBPATH,
